import unittest


class A(unittest.TestCase):
    def test_it(self):
        pass


class B(unittest.TestCase):
    def test_it(self):
        pass


class C(unittest.TestCase):
    def test_it(self):
        pass


class D(unittest.TestCase):
    def test_it(self):
        pass



# -*- encoding:utf-8 -*-
from __future__ import division, print_function, unicode_literals

import random
import time
import sys

from nose.plugins import Plugin
from nose.suite import ContextList

try:
    from factory.fuzzy import set_random_state as factory_set_random_state
    have_factory_boy = True
except ImportError:
    have_factory_boy = False


# Compat
if sys.version_info[0] == 2:  # Python 2
    map_return_type = list
else:
    map_return_type = map


class RandomlyPlugin(Plugin):
    name = 'randomly'

    def options(self, parser, env):
        """Register commandline options.
        """
        super(RandomlyPlugin, self).options(parser, env)
        parser.add_option(
            '--randomly-seed', action='store', dest='seed',
            default=int(time.time()), type=int,
            help="""Set the seed that nose-randomly uses. Default behaviour:
                    use time.time()"""
        )
        parser.add_option(
            '--randomly-dont-shuffle-modules', action='store_false',
            dest='shuffle_modules', default=True,
            help="Stop nose-randomly from shuffling the tests inside modules"
        )
        parser.add_option(
            '--randomly-dont-shuffle-cases', action='store_false',
            dest='shuffle_cases', default=True,
            help="""Stop nose-randomly from shuffling the tests inside TestCase
                    classes"""
        )
        parser.add_option(
            '--randomly-dont-reset-seed', action='store_false',
            dest='reset_seed', default=True,
            help="""Stop nose-randomly from resetting random.seed() at the
                    start of every test."""
        )

    def configure(self, options, conf):
        """
        Configure plugin.
        """
        super(RandomlyPlugin, self).configure(options, conf)

        if not self.enabled:
            return

        self.options = options

    def setOutputStream(self, stream):
        if not self.enabled:
            return

        self.output_stream = stream

        if self.options.reset_seed:
            print(
                "Using --randomly-seed={seed}".format(seed=self.options.seed),
                file=self.output_stream
            )

    def startTest(self, test):
        if not self.enabled:
            return

        if self.options.reset_seed:
            random.seed(self.options.seed)

            if have_factory_boy:
                factory_set_random_state(random.getstate())

    def prepareTestLoader(self, loader):
        """
        Randomize the order of tests loaded from modules and from classes.

        This is a hack. We take the class of the existing, passed in loader
        (normally nose.loader.Loader) and subclass it to monkey-patch in
        shuffle calls for module and case loading when they requested, and then
        mutate the existing test loader's class to this new subclass.

        This is somewhat horrible, but nose's plugin infrastructure isn't so
        flexible - there is no way to wrap just the loader without risking
        interfering with other plugins (if you return anything, no other plugin
        may do anything to the loader).
        """
        if not self.enabled:
            return

        options = self.options

        class ShuffledLoader(loader.__class__):
            def loadTestsFromModule(self, *args, **kwargs):
                """
                Temporarily wrap self.suiteClass with a function that shuffles
                any ContextList instances that the super() call will pass it.
                """
                if options.shuffle_modules:
                    orig_suiteClass = self.suiteClass

                    def hackSuiteClass(tests):
                        if isinstance(tests, ContextList):
                            random.seed(options.seed)
                            random.shuffle(tests.tests)
                        return orig_suiteClass(tests)

                    self.suiteClass = hackSuiteClass
                suite = super(ShuffledLoader, self).loadTestsFromModule(
                    *args, **kwargs)

                if options.shuffle_modules:
                    self.suiteClass = orig_suiteClass

                return suite

            def loadTestsFromTestCase(self, testCaseClass):
                """
                Temporarily wrap self.suiteClass with a function that shuffles
                any list of tests that the super() call will pass it.
                """
                if options.shuffle_cases:
                    orig_suiteClass = self.suiteClass

                    def hackSuiteClass(tests):
                        if isinstance(tests, map_return_type):
                            tests = list(tests)
                            random.seed(options.seed)
                            random.shuffle(tests)
                        return orig_suiteClass(tests)

                    self.suiteClass = hackSuiteClass

                suite = super(ShuffledLoader, self).loadTestsFromTestCase(
                    testCaseClass)

                if options.shuffle_cases:
                    self.suiteClass = orig_suiteClass

                return suite

        # Directly mutate the class of loader... eww
        loader.__class__ = ShuffledLoader

        # Tell the plugin infrastructure we did nothing so 'loader', as mutated
        # above, continues to be used
        return None



import unittest
import random


class Tests(unittest.TestCase):
    def test_not_reseeded_to_100(self):
        numbers = [random.random() for i in range(100)]

        # Now reseed with 100 to find what the first 100 numbers *would* be
        random.seed(100)
        numbers_seed_100 = [random.random() for i in range(100)]

        self.assertNotEqual(numbers, numbers_seed_100)



# -*- coding:utf-8 -*-
from __future__ import unicode_literals

import os
import sys
from unittest import TestCase

from nose.plugins import PluginTester
from nose_randomly import RandomlyPlugin

fixtures = os.path.join(os.path.dirname(__file__), 'fixtures')


class RandomlyPluginTester(PluginTester):
    activate = '--with-randomly'

    def __init__(self, *args, **kwargs):
        super(RandomlyPluginTester, self).__init__(*args, **kwargs)
        self.maxDiff = 10000

    @property
    def plugins(self):
        return [RandomlyPlugin()]

    @property
    def suitepath(self):
        return os.path.join(fixtures, self.fixture_suite)

    def check_output_like(self, lines):
        self.output = list(self.output)
        output = [line.strip() for line in self.output[:len(lines)]]
        self.assertEqual(output, lines)


class ShuffledSubmmodulesInPackageTest(RandomlyPluginTester, TestCase):
    """
    Check that the submodules inside a package are shuffled.
    """
    args = ['-v']
    if sys.version_info >= (3, 0):  # Python 3 random changes
        args.append('--randomly-seed=15')
    else:
        args.append('--randomly-seed=41')

    fixture_suite = 'abcd_package'

    def runTest(self):
        self.check_output_like([
            'Using ' + self.args[-1],
            'test_it (abcd_package.test_d.D) ... ok',
            'test_it (abcd_package.test_c.C) ... ok',
            'test_it (abcd_package.test_a.A) ... ok',
            'test_it (abcd_package.test_b.B) ... ok'
        ])


class ShuffledCasesInModuleTest(RandomlyPluginTester, TestCase):
    """
    Check that the cases inside a module are shuffled.
    """
    args = ['-v']
    if sys.version_info >= (3, 0):  # Python 3 random changes
        args.append('--randomly-seed=38')
    else:
        args.append('--randomly-seed=56')

    fixture_suite = 'abcd_cases.py'

    def runTest(self):
        self.check_output_like([
            'Using ' + self.args[-1],
            'test_it (abcd_cases.C) ... ok',
            'test_it (abcd_cases.A) ... ok',
            'test_it (abcd_cases.B) ... ok',
            'test_it (abcd_cases.D) ... ok'
        ])


class DontShuffledCasesInModuleTest(RandomlyPluginTester, TestCase):
    """
    Check that the cases inside a module are not shuffled when the 'dont' flag
    is set.
    """
    args = ['-v', '--randomly-seed=1', '--randomly-dont-shuffle-modules']
    fixture_suite = 'abcd_cases.py'

    def runTest(self):
        self.check_output_like([
            'Using --randomly-seed=1',
            'test_it (abcd_cases.A) ... ok',
            'test_it (abcd_cases.B) ... ok',
            'test_it (abcd_cases.C) ... ok',
            'test_it (abcd_cases.D) ... ok'
        ])


class ShuffledTestsInTestCaseTest(RandomlyPluginTester, TestCase):
    """
    Check that the tests inside a case are shuffled.
    """
    args = ['-v']
    if sys.version_info >= (3, 0):  # Python 3 random changes
        args.append('--randomly-seed=126')
    else:
        args.append('--randomly-seed=1')

    fixture_suite = 'abcd_tests.py'

    def runTest(self):
        self.check_output_like([
            'Using ' + self.args[-1],
            'test_D (abcd_tests.Tests) ... ok',
            'test_B (abcd_tests.Tests) ... ok',
            'test_C (abcd_tests.Tests) ... ok',
            'test_A (abcd_tests.Tests) ... ok'
        ])


class DontShuffledTestsInTestCaseTest(RandomlyPluginTester, TestCase):
    """
    Check that the tests inside a case are not shuffled when the 'dont' flag is
    set.
    """
    args = ['-v', '--randomly-seed=1', '--randomly-dont-shuffle-cases']
    fixture_suite = 'abcd_tests.py'

    def runTest(self):
        self.check_output_like([
            'Using --randomly-seed=1',
            'test_A (abcd_tests.Tests) ... ok',
            'test_B (abcd_tests.Tests) ... ok',
            'test_C (abcd_tests.Tests) ... ok',
            'test_D (abcd_tests.Tests) ... ok'
        ])


class NotAlwaysOnTests(RandomlyPluginTester, TestCase):
    """
    Check that if we don't activate the plugin using --with-randomly, then it
    doesn't do any shuffling or output.
    """
    activate = '-v'  # Abuse of this to fill in args - it can't be None :(
    fixture_suite = 'abcd_tests.py'

    def runTest(self):
        self.check_output_like([
            'test_A (abcd_tests.Tests) ... ok',
            'test_B (abcd_tests.Tests) ... ok',
            'test_C (abcd_tests.Tests) ... ok',
            'test_D (abcd_tests.Tests) ... ok'
        ])


class RandomSeedTest(RandomlyPluginTester, TestCase):
    """
    Check that the random seed is being set.
    """
    args = ['-v', '--randomly-seed=1']
    fixture_suite = 'random_number.py'

    def runTest(self):
        # Just runs the test - the remaining logic is in the file itself,
        # checking that random.random() gives result it should when seed = 1
        self.check_output_like([
            'Using --randomly-seed=1',
            'test_random (random_number.Tests) ... ok'
        ])


class DontRandomSeedTest(RandomlyPluginTester, TestCase):
    """
    Check that the random seed is being set.
    """
    args = ['-v', '--randomly-dont-reset-seed']
    fixture_suite = 'random_seed_not_100.py'

    def runTest(self):
        # Just runs the test - the remaining logic is in the file itself,
        # checking that random.random() does not look like the seed is 100
        self.check_output_like([
            'test_not_reseeded_to_100 (random_seed_not_100.Tests) ... ok'
        ])
